;; Generated package description from ef-themes.el  -*- no-byte-compile: t -*-
(define-package "ef-themes" "0.3.4" "Colorful and legible themes" '((emacs "27.1")) :commit "dc35c4fdbdce3a3656177024e9be708def0122ea" :authors '(("Protesilaos Stavrou" . "info@protesilaos.com")) :maintainer '("Ef-Themes Development" . "~protesilaos/ef-themes@lists.sr.ht") :keywords '("faces" "theme" "accessibility") :url "https://git.sr.ht/~protesilaos/ef-themes")
